@@ -82,10 +82,6 @@ RequestAnswer::RequestAnswer(ParsedMessage&& msg)
    nodes6(std::move(msg.nodes6))
 {}
 
-NetworkEngine::NetworkEngine(const Sp<Logger>& log, std::mt19937_64& rand, Scheduler& scheduler, std::unique_ptr<DatagramSocket>&& sock)
-    : myid(zeroes), dht_socket(std::move(sock)), logger_(log), rd(rand), cache(rd), rate_limiter((size_t)-1), scheduler(scheduler)
-{}
-
 NetworkEngine::NetworkEngine(InfoHash& myid, NetworkConfig c,
         std::unique_ptr<DatagramSocket>&& sock,
         const Sp<Logger>& log,
@@ -380,7 +376,7 @@ NetworkEngine::isMartian(const SockAddr& addr)
         const uint8_t* address = (const uint8_t*)&sin6.sin6_addr;
         return address[0] == 0xFF ||
               (address[0] == 0xFE && (address[1] & 0xC0) == 0x80) ||
-               memcmp(address, zeroes.data(), 16) == 0 ||
+               memcmp(address, InfoHash::zero().data(), 16) == 0 ||
                memcmp(address, v4prefix,      12) == 0;
     }
     default:
@@ -460,9 +456,13 @@ NetworkEngine::processMessage(const uint8_t *buf, size_t buflen, SockAddr f)
             pmsg_it->second.last_part = now;
             // check data completion
             if (pmsg_it->second.msg->complete()) {
-                // process the full message
-                process(std::move(pmsg_it->second.msg), from);
-                partial_messages.erase(pmsg_it);
+                try {
+                    // process the full message
+                    process(std::move(pmsg_it->second.msg), from);
+                    partial_messages.erase(pmsg_it);
+		} catch (...) {
+                    return;
+		}  
             } else
                 scheduler.add(now + RX_TIMEOUT, std::bind(&NetworkEngine::maintainRxBuffer, this, msg->tid));
         }
@@ -485,7 +485,11 @@ NetworkEngine::processMessage(const uint8_t *buf, size_t buflen, SockAddr f)
     }
 
     if (msg->value_parts.empty()) {
-        process(std::move(msg), from);
+        try {	    
+            process(std::move(msg), from);
+	} catch(...) {
+            return;
+	}
     } else {
         // starting partial message session
         auto k = msg->tid;
