@@ -33,7 +33,7 @@ CryptoTester::setUp() {
 void
 CryptoTester::testSignatureEncryption() {
     auto key = dht::crypto::PrivateKey::generate();
-    auto public_key = key.getPublicKey();
+    const auto& public_key = key.getPublicKey();
 
     std::vector<uint8_t> data1 {5, 10};
     std::vector<uint8_t> data2(64 * 1024, 10);
@@ -44,6 +44,10 @@ CryptoTester::testSignatureEncryption() {
     // check signature
     CPPUNIT_ASSERT(public_key.checkSignature(data1, signature1));
     CPPUNIT_ASSERT(public_key.checkSignature(data2, signature2));
+    signature1[7]++;
+    signature2[8]--;
+    CPPUNIT_ASSERT(!public_key.checkSignature(data1, signature1));
+    CPPUNIT_ASSERT(!public_key.checkSignature(data2, signature2));
 
     // encrypt data
     {
@@ -161,6 +165,14 @@ void CryptoTester::testCertificateSerialNumber()
     CPPUNIT_ASSERT(std::equal(SERIAL.begin(), SERIAL.end(), serial.begin(), serial.end()));
 }
 
+void CryptoTester::testOcsp() {
+    auto ca = dht::crypto::generateIdentity("Test CA");
+    auto device = dht::crypto::generateIdentity("Test Device", ca);
+    auto ocspRequest = device.second->generateOcspRequest(ca.second->cert);
+    auto req = dht::crypto::OcspRequest((const uint8_t*)ocspRequest.first.data(), ocspRequest.first.size());
+    CPPUNIT_ASSERT(ocspRequest.second == req.getNonce());
+}
+
 void
 CryptoTester::tearDown() {
 
